package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.model.account.User
import io.jsonwebtoken.Jwts
import io.jsonwebtoken.SignatureAlgorithm
import io.jsonwebtoken.io.Encoders
import io.jsonwebtoken.security.Keys
import java.util.*
import javax.crypto.SecretKey

data class Jwt(
    val subject: String,
    val secret: String,
    val duration: Long? = null,
    val signatureAlgorithm: SignatureAlgorithm = SignatureAlgorithm.HS512
) {
    val token: String by lazy {
        val builder = Jwts.builder()
            .setSubject(subject)

        duration?.let {
            val expirationMs = System.currentTimeMillis() + it
            val expirationDate = Date(expirationMs)

            builder.setExpiration(expirationDate)
        }

        builder
            .signWith(keyFromSecret(secret))
            .compact()
    }
}

/** Build a [Jwt] for the given [User]. */
fun User.buildJwt(secret: String, duration: Long?) =
    Jwt(this.id.toString(), secret, duration)

/** Parses the given [jwt] string. */
fun parseJwt(jwt: String, secret: String) =
    with(
        Jwts.parserBuilder()
            .setSigningKey(keyFromSecret(secret))
            .build()
            .parseClaimsJws(jwt)
    ) {
        Jwt(this.body.subject, secret)
    }

/** Creates a base64 encoded [SecretKey] from the given [secret]. */
private fun keyFromSecret(secret: String) =
    with(Encoders.BASE64.encode(secret.toByteArray())) {
        Keys.hmacShaKeyFor(this.toByteArray())
    }
